"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const request = require('request-promise-native');
const error_1 = require("../../utils/error");
const bitbucketurl = `${global.bitbucket}/rest/api/1.0/`;
const Authorization = global.bitbucketKey;
const { internalError } = error_1.default('Bitbucket API call failed: ');
exports.Bitbucket = {
    get: (url) => apiCall({ url, method: 'get' }),
    put: (url, body) => apiCall({ method: 'put', url, body }),
    post: (url, body) => apiCall({ method: 'post', url, body }),
};
function apiCall({ url, method, body }) {
    const config = {
        uri: bitbucketurl + url,
        method,
        json: true,
        headers: {
            Authorization,
        },
    };
    if (body) {
        config.body = body;
    }
    return request(config).catch((err) => {
        internalError(2, `Error sending ${method}-request to ${url} with payload:\n ${JSON.stringify(body, null, 2)}`)(err);
        return Promise.reject('Error while accessing Bitbucket API.');
    });
}
