"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tasks_1 = require("./tasks");
const bitbucket_1 = require("./bitbucket");
const comments_1 = require("../db/comments");
const create_comment_1 = require("../../utils/parsing/create-comment");
const diff_1 = require("../../utils/parsing/diff");
function createApiSlug({ repository: { type, repo, project }, name }, commentId) {
    return `${type}/${project}/repos/${repo}/pull-requests/${name}/comments/${commentId || ''}`;
}
async function writeReportToBitbucket(base, pr) {
    const diff = diff_1.diffReports(base.reports[0], pr.reports[0]);
    const existingComment = await comments_1.commentDb.get(pr.repository, pr.name);
    const comment = create_comment_1.createCommentObject(diff, existingComment, pr.lcov);
    const { id, version } = existingComment
        ? await bitbucket_1.Bitbucket.get(createApiSlug(pr, existingComment.commentId))
            .then(({ version, id }) => bitbucket_1.Bitbucket.put(createApiSlug(pr, id), { ...comment, version }))
        : await bitbucket_1.Bitbucket.post(createApiSlug(pr), comment);
    const newComment = {
        ...existingComment,
        commentId: id.toString(),
        version,
    };
    if (pr.task && diff.total.diff.statementCov < 0) {
        newComment.taskId = await tasks_1.openTask(newComment);
    }
    else if (newComment.taskId && diff.total.diff.statementCov >= 0) {
        await tasks_1.resolveTask(newComment);
    }
    return comments_1.commentDb.set(pr.repository, pr.name, newComment);
}
exports.writeReportToBitbucket = writeReportToBitbucket;
