"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const forge_1 = require("../../utils/forge");
const file_db_1 = require("../../utils/file-db");
const writeDiffs = (id, data) => fs_extra_1.outputFile(global.storage + `branches/branch_${id}.json`, JSON.stringify(data), 'utf8');
const readDiffs = id => fs_extra_1.readJSON(global.storage + `branches/branch_${id}.json`, 'utf8');
class BranchDb {
    constructor() {
        this.ready = file_db_1.default('branches.json').then((db) => {
            this.db = db;
        });
        this.get = this.get.bind(this);
        this.create = this.create.bind(this);
        this.update = this.update.bind(this);
    }
    async exists(repository, name) {
        await this.ready;
        const id = forge_1.forge.branchId({ repository, name });
        return !!this.db.get(id);
    }
    async get(repository, name) {
        await this.ready;
        const id = forge_1.forge.branchId({ repository, name });
        const branch = this.db.get(id);
        if (branch) {
            const reports = await readDiffs(id);
            return { ...branch, reports };
        }
        return null;
    }
    async update(branchRest) {
        await this.ready;
        if (!(await this.exists(branchRest.repository, branchRest.name)))
            return Promise.reject('Branch does not exist');
        const id = forge_1.forge.branchId(branchRest);
        const reports = (await readDiffs(id)).filter(({ timestamp }) => timestamp !== branchRest.report.timestamp);
        const branch = this.db.get(id);
        branch.lastModified = Date.now();
        await Promise.all([writeDiffs(id, [branchRest.report, ...reports]), this.db.set(id, branch)]);
        return { ...branch, reports: [branchRest.report, ...reports] };
    }
    async create(branchRest) {
        await this.ready;
        if (await this.exists(branchRest.repository, branchRest.name))
            return Promise.reject('Branch already exists');
        const id = forge_1.forge.branchId(branchRest);
        const branch = {
            repository: branchRest.repository,
            name: branchRest.name,
            id,
            lastModified: Date.now(),
        };
        await Promise.all([writeDiffs(id, [branchRest.report]), this.db.set(id, branch)]);
        return { ...branch, reports: [branchRest.report] };
    }
}
exports.branchDb = new BranchDb();
