"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const forge_1 = require("../../utils/forge");
const file_db_1 = require("../../utils/file-db");
const writeDiffs = (id, data) => fs_extra_1.outputFile(global.storage + `pullRequests/pullRequest_${id}.json`, JSON.stringify(data), 'utf8');
const readDiffs = id => fs_extra_1.readJSON(global.storage + `pullRequests/pullRequest_${id}.json`, 'utf8');
class PullRequestDb {
    constructor() {
        this.ready = file_db_1.default('pullRequests.json').then((db) => {
            this.db = db;
        });
        this.get = this.get.bind(this);
        this.create = this.create.bind(this);
        this.update = this.update.bind(this);
    }
    async exists(repository, name) {
        await this.ready;
        const id = forge_1.forge.pullRequestId({ repository, name });
        return !!this.db.get(id);
    }
    async get(repository, name) {
        await this.ready;
        const id = forge_1.forge.pullRequestId({ repository, name });
        const pullRequest = this.db.get(id);
        if (pullRequest) {
            const reports = await readDiffs(id);
            return { ...pullRequest, reports };
        }
        return null;
    }
    async update({ name, repository, report, task, lcov }) {
        await this.ready;
        if (!(await this.exists(repository, name)))
            return Promise.reject('Pull request does not exist');
        const id = forge_1.forge.pullRequestId({ repository, name });
        const reports = (await readDiffs(id)).filter(({ timestamp }) => timestamp !== report.timestamp);
        await Promise.all([
            this.db.set(id, { ...this.db.get(id), lastModified: Date.now(), task, lcov }),
            writeDiffs(id, [report, ...reports]),
        ]);
        return { ...this.db.get(id), reports: [report, ...reports] };
    }
    async create({ name, repository, report, task, lcov }, base) {
        await this.ready;
        if (await this.exists(repository, name))
            return Promise.reject('Pull request already exists');
        const id = forge_1.forge.pullRequestId({ repository, name });
        const pullRequest = {
            id,
            name,
            repository,
            base,
            task,
            lcov,
            lastModified: Date.now(),
        };
        await Promise.all([writeDiffs(id, [report]), this.db.set(id, pullRequest)]);
        return { ...pullRequest, reports: [report] };
    }
}
exports.pullRequestDb = new PullRequestDb();
