"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const branches_1 = require("../modules/db/branches");
const error_1 = require("../utils/error");
const get_base_branch_1 = require("../modules/bitbucket/get-base-branch");
const pull_requests_1 = require("../modules/db/pull-requests");
const comment_1 = require("../modules/bitbucket/comment");
const { routerError, internalError } = error_1.default('controllers:');
exports.addBranchReport = async (req, res) => {
    const { name, repository, report } = req.body;
    const action = (await branches_1.branchDb.exists(repository, name)) ? branches_1.branchDb.update : branches_1.branchDb.create;
    action({ name, repository, report })
        .then(() => {
        res.status(200).send({ success: true });
    })
        .catch(routerError(2, res, 'Error adding branch report.'));
};
exports.addPullRequestReport = async (req, res) => {
    try {
        const pullRequestRest = { ...req.body, task: !!req.body.task };
        const action = (await pull_requests_1.pullRequestDb.exists(pullRequestRest.repository, pullRequestRest.name))
            ? pull_requests_1.pullRequestDb.update
            : pull_requests_1.pullRequestDb.create;
        const base = await get_base_branch_1.getBaseBranchFromPullrequest(pullRequestRest);
        const baseBranch = await branches_1.branchDb.get(base.repository, base.name);
        if (!baseBranch) {
            throw 'Base branch is not in the database.';
        }
        const pr = await action(pullRequestRest, base);
        await comment_1.writeReportToBitbucket(baseBranch, pr).catch(internalError(2, 'Error handling pull request:'));
        res.status(200).send({ success: true });
    }
    catch (err) {
        routerError(2, res, 'Error adding pullrequest report.')(err);
    }
};
