"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
/**
 *   Log Levels:
 *   0: no error, very important info
 *   1: error: critical error in app
 *   2: error: internal error in app
 *   3: error: minor error
 *
 *   4: info: important info (bad request/login)
 *   5: info: minor info
 *
 *   6: trace: tracing informations
 *   7: unimportant tracing information
 *
 *  10: full trace: (requests objects, etc)
 *
 **/
let currentDate = new Date().toISOString().split('T')[0];
if (!fs_1.existsSync(global.logFolder)) {
    fs_1.mkdirSync(global.logFolder);
}
let logStream = fs_1.createWriteStream(global.logFolder + '/covwatcher_' + currentDate + '.log', {
    flags: 'a',
    encoding: 'utf8',
    autoClose: true,
});
logStream.on('error', err => {
    console.log(err);
});
exports.default = (level, ...message) => {
    const now = new Date().toISOString();
    const messages = message.map(parseMessage).join(' ');
    const logMessage = `${now} - ${level} - ${messages}`;
    if (level <= global.logLevel) {
        console.log(logMessage);
    }
    if (currentDate !== new Date().toISOString().split('T')[0]) {
        currentDate = new Date().toISOString().split('T')[0];
        logStream.end();
        logStream = fs_1.createWriteStream(global.logFolder + '/covwatcher_' + currentDate + '.log', {
            flags: 'a',
            encoding: 'utf8',
            autoClose: true,
        });
    }
    logStream.write(logMessage + '\n');
};
function parseMessage(item) {
    let output;
    if (['string', 'number', 'boolean'].includes(typeof item)) {
        output = item;
    }
    else {
        try {
            if (Object.prototype.toString.call(item) === '[object Error]') {
                output = '\n' + item.stack.toString();
            }
            else {
                output = '\n' + JSON.stringify(item);
            }
        }
        catch (err) {
            console.log(err + '\n' + item);
            output = '';
        }
    }
    return output;
}
